@@ -47,7 +47,9 @@ preload_map_t *
 preload_map_new (const char *path, size_t offset, size_t length)
 {
   preload_map_t *map;
-  g_assert (path);
+
+  g_return_val_if_fail (path, NULL);
+
   map = g_malloc (sizeof (*map));
   map->path = g_strdup (path);
   map->offset = offset;
@@ -61,15 +63,37 @@ preload_map_new (const char *path, size_t offset, size_t length)
 void
 preload_map_free (preload_map_t *map)
 {
-  g_assert (map);
-  g_assert (map->refcount == 0);
-  g_assert (map->path);
+  g_return_if_fail (map);
+  g_return_if_fail (map->refcount == 0);
+  g_return_if_fail (map->path);
+
   g_free (map->path);
   map->path = NULL;
   g_free (map);
 }
 
 
+static void
+preload_state_register_map (preload_map_t *map)
+{
+  g_return_if_fail (!g_hash_table_lookup (state->maps, map));
+
+  map->seq = ++(state->map_seq);
+  g_hash_table_insert (state->maps, map, GINT_TO_POINTER (1));
+  g_ptr_array_add (state->maps_arr, map);
+}
+
+
+static void
+preload_state_unregister_map (preload_map_t *map)
+{
+  g_return_if_fail (g_hash_table_lookup (state->maps, map));
+
+  g_ptr_array_remove (state->maps_arr, map);
+  g_hash_table_remove (state->maps, map);
+}
+
+
 void
 preload_map_ref (preload_map_t *map)
 {
@@ -82,8 +106,9 @@ preload_map_ref (preload_map_t *map)
 void
 preload_map_unref (preload_map_t *map)
 {
-  g_assert (map);
-  g_assert (map->refcount > 0);
+  g_return_if_fail (map);
+  g_return_if_fail (map->refcount > 0);
+
   map->refcount--;
   if (!map->refcount) {
     preload_state_unregister_map (map);
@@ -95,7 +120,8 @@ preload_map_unref (preload_map_t *map)
 size_t
 preload_map_get_size (preload_map_t *map)
 {
-  g_assert (map);
+  g_return_val_if_fail (map, 0);
+
   return map->length;
 }
 
@@ -103,8 +129,9 @@ preload_map_get_size (preload_map_t *map)
 guint
 preload_map_hash (preload_map_t *map)
 {
-  g_assert (map);
-  g_assert (map->path);
+  g_return_val_if_fail (map, 0);
+  g_return_val_if_fail (map->path, 0);
+
   return g_str_hash (map->path)
        + g_direct_hash (GSIZE_TO_POINTER (map->offset))
        + g_direct_hash (GSIZE_TO_POINTER (map->length));
@@ -124,7 +151,9 @@ preload_exemap_t *
 preload_exemap_new (preload_map_t *map)
 {
   preload_exemap_t *exemap;
-  g_assert (map);
+
+  g_return_val_if_fail (map, NULL);
+
   preload_map_ref (map);
   exemap = g_malloc (sizeof (*exemap));
   exemap->map = map;
@@ -136,9 +165,10 @@ preload_exemap_new (preload_map_t *map)
 void
 preload_exemap_free (preload_exemap_t *exemap)
 {
-  g_assert (exemap);
-  g_assert (exemap->map);
-  preload_map_unref (exemap->map);
+  g_return_if_fail (exemap);
+
+  if (exemap->map)
+    preload_map_unref (exemap->map);
   g_free (exemap);
 }
 
@@ -177,9 +207,11 @@ preload_markov_t *
 preload_markov_new (preload_exe_t *a, preload_exe_t *b, gboolean initialize)
 {
   preload_markov_t *markov;
-  g_assert (a);
-  g_assert (b);
-  g_assert (a != b);
+
+  g_return_val_if_fail (a, NULL);
+  g_return_val_if_fail (b, NULL);
+  g_return_val_if_fail (a != b, NULL);
+
   markov = g_malloc (sizeof (*markov));
   markov->a = a;
   markov->b = b;
@@ -221,7 +253,7 @@ preload_markov_state_changed (preload_markov_t *markov)
   old_state = markov->state;
   new_state = markov_state (markov);
 
-  g_assert (old_state != new_state);
+  g_return_if_fail (old_state != new_state);
 
   markov->weight[old_state][old_state]++;
   markov->time_to_leave[old_state] += ((state->time - markov->change_timestamp)
@@ -237,7 +269,8 @@ preload_markov_state_changed (preload_markov_t *markov)
 void
 preload_markov_free (preload_markov_t *markov, preload_exe_t *from)
 {
-  g_assert (markov);
+  g_return_if_fail (markov);
+
   if (from) {
     preload_exe_t *other;
     g_assert (markov->a == from || markov->b == from);
@@ -355,7 +388,9 @@ preload_exe_t *
 preload_exe_new (const char *path, gboolean running, GSet *exemaps)
 {
   preload_exe_t *exe;
-  g_assert (path);
+
+  g_return_val_if_fail (path, NULL);
+
   exe = g_malloc (sizeof (*exe));
   exe->path = g_strdup (path);
   exe->size = 0;
@@ -379,8 +414,9 @@ preload_exe_new (const char *path, gboolean running, GSet *exemaps)
 void
 preload_exe_free (preload_exe_t *exe)
 {
-  g_assert (exe);
-  g_assert (exe->path);
+  g_return_if_fail (exe);
+  g_return_if_fail (exe->path);
+
   g_set_foreach (exe->exemaps, (GFunc)preload_exemap_free, NULL);
   g_set_free (exe->exemaps);
   exe->exemaps = NULL;
@@ -397,8 +433,10 @@ preload_exemap_t *
 preload_exe_map_new (preload_exe_t *exe, preload_map_t *map)
 {
   preload_exemap_t *exemap;
-  g_assert (exe);
-  g_assert (map);
+
+  g_return_val_if_fail (exe, NULL);
+  g_return_val_if_fail (map, NULL);
+
   exemap = preload_exemap_new (map);
   g_set_add (exe->exemaps, exemap);
   exe_add_map_size (exemap, exe);
@@ -408,25 +446,6 @@ preload_exe_map_new (preload_exe_t *exe, preload_map_t *map)
 
 
 
-void
-preload_state_register_map (preload_map_t *map)
-{
-  g_assert (!g_hash_table_lookup (state->maps, map));
-  map->seq = ++(state->map_seq);
-  g_hash_table_insert (state->maps, map, GINT_TO_POINTER (1));
-  g_ptr_array_add (state->maps_arr, map);
-}
-
-
-void
-preload_state_unregister_map (preload_map_t *map)
-{
-  g_assert (g_hash_table_lookup (state->maps, map));
-  g_ptr_array_remove (state->maps_arr, map);
-  g_hash_table_remove (state->maps, map);
-}
-
-
 static void
 shift_preload_markov_new (gpointer G_GNUC_UNUSED key, preload_exe_t *a, preload_exe_t *b)
 {
@@ -438,7 +457,8 @@ shift_preload_markov_new (gpointer G_GNUC_UNUSED key, preload_exe_t *a, preload_
 void
 preload_state_register_exe (preload_exe_t *exe, gboolean create_markovs)
 {
-  g_assert (!g_hash_table_lookup (state->exes, exe));
+  g_return_if_fail (!g_hash_table_lookup (state->exes, exe));
+
   exe->seq = ++(state->exe_seq);
   if (create_markovs) {
     g_hash_table_foreach (state->exes, (GHFunc)shift_preload_markov_new, exe);
@@ -450,7 +470,8 @@ preload_state_register_exe (preload_exe_t *exe, gboolean create_markovs)
 void
 preload_state_unregister_exe (preload_exe_t *exe)
 {
-  g_assert (g_hash_table_lookup (state->exes, exe));
+  g_return_if_fail (g_hash_table_lookup (state->exes, exe));
+
   g_set_foreach (exe->markovs, (GFunc)preload_markov_free, exe);
   g_set_free (exe->markovs);
   exe->markovs = NULL;
@@ -480,6 +501,7 @@ preload_state_unregister_exe (preload_exe_t *exe)
 #define READ_SYNTAX_ERROR		"invalid syntax"
 #define READ_INDEX_ERROR		"invalid index"
 #define READ_DUPLICATE_INDEX_ERROR	"duplicate index"
+#define READ_DUPLICATE_OBJECT_ERROR	"duplicate object"
 
 typedef struct _read_context_t
 {
@@ -502,11 +524,11 @@ read_map (read_context_t *rc)
   preload_map_t *map;
   int update_time;
   int i, expansion;
-  size_t offset, length;
+  long offset, length;
   char *path;
 
   if (6 > sscanf (rc->line,
-		  "%d %d %u %u %d %"FILELENSTR"s",
+		  "%d %d %lu %lu %d %"FILELENSTR"s",
 		  &i, &update_time, &offset, &length, &expansion, rc->filebuf)) {
     rc->errmsg = READ_SYNTAX_ERROR;
     return;
@@ -522,6 +544,10 @@ read_map (read_context_t *rc)
     rc->errmsg = READ_DUPLICATE_INDEX_ERROR;
     return;
   }
+  if (g_hash_table_lookup (state->maps, map)) {
+    rc->errmsg = READ_DUPLICATE_OBJECT_ERROR;
+    return;
+  }
 
   map->update_time = update_time;
   preload_map_ref (map);
@@ -581,6 +607,10 @@ read_exe (read_context_t *rc)
     rc->errmsg = READ_DUPLICATE_INDEX_ERROR;
     return;
   }
+  if (g_hash_table_lookup (state->exes, exe->path)) {
+    rc->errmsg = READ_DUPLICATE_OBJECT_ERROR;
+    return;
+  }
 
   exe->update_time = update_time;
   exe->time = time;
@@ -881,8 +911,8 @@ static void write_map (preload_map_t *map, gpointer G_GNUC_UNUSED data, write_co
 
   write_tag (TAG_MAP);
   g_string_printf (wc->line,
-		   "%d\t%d\t%u\t%u\t%d\t%s",
-		   map->seq, map->update_time, map->offset, map->length, -1/*expansion*/, uri);
+		   "%d\t%d\t%lu\t%lu\t%d\t%s",
+		   map->seq, map->update_time, (long)map->offset, (long)map->length, -1/*expansion*/, uri);
   write_string (wc->line);
   write_ln ();
 
